#!/bin/bash
#
# cd-names.sh
#
# (Description shamelessly borrowed from cdsymlinks.sh)
#
# Maps CDROM, CD-R, CDRW, DVD, DVDRW, DVDRAM and Mountrainer types of
# devices to its udev name and synonyms.
# First parameter is the kernel device name (e.g. hdc or sr0)
#
# Usage: 
# BUS="ide", KERNEL="hd[a-z]", PROGRAM="/etc/udev/scripts/cd-names.sh %k", NAME="%c{1}", SYMLINK="%c{2+}"
# BUS="scsi", KERNEL="sr[0-9]*", PROGRAM="/etc/udev/scripts/cd-names.sh %k", NAME="%c{1}", SYMLINK="%c{2+}"
# BUS="scsi", KERNEL="scd[0-9]*", PROGRAM="/etc/udev/scripts/cd-names.sh %k", NAME="%c{1}", SYMLINK="%c{2+}"
# (this last one is "just in case")
#
# Additional configuration is available through /etc/udev/cd-names.conf
# See comments in that file regarding configuration parameters and
# their meanings
# 
# (c) 2004, Alexey Morozov <morozov@altlinux.org>
#
# BUGS: Default configuration relies on %N and %U keys supported
# by udev (Patches needed as of udev-046)
# 
# I'm not sure if this script is sh-portable.
# But it works at least in bash2 ;-)

CDDRIVE_INFO=/proc/sys/dev/cdrom/info
[ -r "$CDDRIVE_INFO" ] || exit 1

test -r /etc/udev/cd-names.conf && . /etc/udev/cd-names.conf
: ${DRIVETYPE_PRIO:=DVDRW CDRW DVD CD}
: ${DO_ALL_LINKS:=1}
: ${DO_ROOT_LINKS:=1}
: ${PRIMARY_SUFFIX:=%n}
: ${SECONDARY_SUFFIX:=%e}
: ${DEBUG:=0}

## use a wrapper instead of a raw grep. Useful for future
# switch to a pure-shell substitution
function do_grep() {
    grep "$@"
    return $?
}

function dprintf() {
    test "x$DEBUG" = "x1" && printf "$@" >&2
}

function decho() {
    test "x$DEBUG" = "x1" && echo "$@" >&2
}

function die() {
    printf "$@" >&2
    exit 1
}

function getDriveColumn() {
    local driveName driveString entireString rc
    dprintf "getDriveColumn(%s)\n" "$1"
    driveName=$1
    test -n "$driveName" || die "empty driveName passed\n"
    driveString="drive name:"
    entireString=`do_grep -- "^$driveString" $CDDRIVE_INFO`
    rc=$?
    test "$rc" = 0 || return -1
    entireString=${entireString:${#driveString}}
    dprintf "getDriveColumn() entireString=\"%s\"\n" "$entireString"
    test -n "$entireString" || return 0
    set -- $entireString ''
    column=1
    rc=0
    while [ $# -gt 1 ]; do
	decho "getDriveColumn() checking" "$1"
	test "x$1" = "x$driveName" && rc=$column && break
	column=$(($column + 1))
	shift
    done
    decho "getDriveColumn() rc=$rc"
    return $rc
}

function getCapabilityInt()
{
    dprintf "getCapabilityInt(%d,\"%s\")\n" "$1" "$2"
    local capString dc entireString rc
    dc=$1
    capString=$2
    test -n "$dc" -a -n "$capString" || die "both Drive Column and Capability String should be specified\n"
    entireString=`do_grep -- "^$capString" $CDDRIVE_INFO`
    rc=$?
    test "$rc" != 0 -o -z "$entireString" && return 0
    entireString=${entireString:${#capString}}
    dprintf "getCapabilityInt() entireString=\"%s\"\n" "$entireString"
    set -- $entireString ''
    # check if enough columns were returned. Return 0 if it's not
    # (as if drive had no such capability)
    test $# -lt $dc && return 0
    eval "rc=\$$dc"
    return $rc
}

function getCapability()
{
    dprintf "getCapabilityInt(%d,\"%s\")\n" "$1" "$2"
    local capName dc
    dc=$1
    capName=$2
    test -n "$dc" -a -n "$capName" || die "both Drive Column and Capability Name should be specified\n"
    local CDT_CDWR_T CDT_CDWRW_T CDT_DVD_T CDT_DVDWR_T CDT_DVDWRAM_T CDT_MRW_T CDT_MRWW_T CDT_RAMW_T
    CDT_CDWR_T="Can write CD-R:"
    CDT_CDWRW_T="Can write CD-RW:"
    CDT_DVD_T="Can read DVD:"
    CDT_DVDWR_T="Can write DVD-R:"
    CDT_DVDWRAM_T="Can write DVD-RAM:"
    CDT_MRW_T="Can read MRW:"
    CDT_MRWW_T="Can write MRW:"
    CDT_RAMW_T="Can write RAM:"
    # device classes
    local CDC_CDRW CDC_DVD CDC_DVDRW
    CDC_CDRW="CDWR CDWRW MRWW RAMW"
    CDC_DVD="DVD"
    CDC_DWDRW="DVDWR DVDWRAM"
    CDC_CD="CD"
    local title class rc classes 
    # A dirty hack. Well, you know, it's shell programming anyway.
    for cltype in `eval "echo \\$CDC_$capName"`; do
	# Special case: if cltype is CD then we automatically return 1
	test "x$cltype" = "xCD" && return 1
	title=
	eval "title=\$CDT_${cltype}_T"
	decho "getCapability() cltype='$cltype', title='$title'"
	test -n "$title" || continue
	getCapabilityInt "$driveColumn" "$title" || return 1
    done
    return 0
}

function reportDrive() {
    dprintf "reportDrive(\"%s\")\n" "$1"
    local driveType
    driveType=$1
    local CDNAME_CD CDNAME_CDRW CDNAME_DVD CDNAME_DVDRW
    # Canonical names for each class of CD/DVD devices
    : ${CDNAME_CD:="cdrom"}
    : ${CDNAME_CDRW:="cdwriter"}
    : ${CDNAME_DVD:="dvd"}
    : ${CDNAME_DVDRW:="dvdwriter"}
    local driveName
    driveName=
    eval "driveName=\$CDNAME_${driveType}"
    test -n "$driveName" || die "Unknown drive type '$driveType' specified\n"
    echo -n " cdroms/${driveName}${PRIMARY_SUFFIX}"
    test "x$DO_ROOT_LINKS" = "x1" && echo -n " ${driveName}${SECONDARY_SUFFIX}"
    return 0
}

declare driveColumn

function driveCan()
{
    getCapability $driveColumn "$@" || return 0
    return 1
}

declare driveName

driveName=$1
getDriveColumn $driveName
driveColumn=$?
test $driveColumn -gt 0 || die "Can't calculate drive '%s' column (rc=%d)\n" "$driveName" "$driveColumn"

# Finally report all drive names

# Add CD-ROM type if it's not in the list yet
DRIVETYPE_PRIO=" $DRIVETYPE_PRIO "
test "x${DRIVETYPE_PRIO/ CD }" == "x$DRIVETYPE_PRIO" && \
    DRIVETYPE_PRIO="$DRIVETYPE_PRIO CD"

# Now loop through all "extra" drive types
declare drivetype
for drivetype in $DRIVETYPE_PRIO; do
    if driveCan $drivetype; then
	reportDrive $drivetype
	test "x$DO_ALL_LINKS" != "x1" && break
    fi
done

echo
